@@ -1,42 +1,36 @@
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import List
-from typing import Optional
+from __future__ import annotations
 
-from .empty_constraint import EmptyConstraint
-from .version_constraint import VersionConstraint
-from .version_union import VersionUnion
+from typing import TYPE_CHECKING
+
+from poetry.core.semver.empty_constraint import EmptyConstraint
+from poetry.core.semver.version_range_constraint import VersionRangeConstraint
+from poetry.core.semver.version_union import VersionUnion
 
 
 if TYPE_CHECKING:
     from poetry.core.semver.version import Version
-
-    from . import VersionTypes  # noqa
+    from poetry.core.semver.version_constraint import VersionConstraint
 
 
-class VersionRange(VersionConstraint):
+class VersionRange(VersionRangeConstraint):
     def __init__(
         self,
-        min=None,  # type: Optional["Version"]
-        max=None,  # type: Optional["Version"]
-        include_min=False,  # type: bool
-        include_max=False,  # type: bool
-        always_include_max_prerelease=False,  # type: bool
-    ):
+        min: Version | None = None,
+        max: Version | None = None,
+        include_min: bool = False,
+        include_max: bool = False,
+        always_include_max_prerelease: bool = False,
+    ) -> None:
         full_max = max
         if (
             not always_include_max_prerelease
             and not include_max
             and full_max is not None
-            and not full_max.is_prerelease()
-            and not full_max.build
-            and (
-                min is None
-                or not min.is_prerelease()
-                or not min.equals_without_prerelease(full_max)
-            )
+            and full_max.is_stable()
+            and not full_max.is_postrelease()
+            and (min is None or min.is_stable() or min.release != full_max.release)
         ):
-            full_max = full_max.first_prerelease
+            full_max = full_max.first_prerelease()
 
         self._min = min
         self._max = max
@@ -45,32 +39,35 @@ class VersionRange(VersionConstraint):
         self._include_max = include_max
 
     @property
-    def min(self):  # type: () -> "Version"
+    def min(self) -> Version | None:
         return self._min
 
     @property
-    def max(self):  # type: () -> "Version"
+    def max(self) -> Version | None:
         return self._max
 
     @property
-    def full_max(self):  # type: () -> "Version"
+    def full_max(self) -> Version | None:
         return self._full_max
 
     @property
-    def include_min(self):  # type: () -> bool
+    def include_min(self) -> bool:
         return self._include_min
 
     @property
-    def include_max(self):  # type: () -> bool
+    def include_max(self) -> bool:
         return self._include_max
 
-    def is_empty(self):  # type: () -> bool
+    def is_empty(self) -> bool:
         return False
 
-    def is_any(self):  # type: () -> bool
+    def is_any(self) -> bool:
         return self._min is None and self._max is None
 
-    def allows(self, other):  # type: ("Version") -> bool
+    def is_simple(self) -> bool:
+        return self._min is None or self._max is None
+
+    def allows(self, other: Version) -> bool:
         if self._min is not None:
             if other < self._min:
                 return False
@@ -79,16 +76,26 @@ class VersionRange(VersionConstraint):
                 return False
 
         if self.full_max is not None:
-            if other > self.full_max:
+            _this, _other = self.full_max, other
+
+            if not _this.is_local() and _other.is_local():
+                # allow weak equality to allow `3.0.0+local.1` for `<=3.0.0`
+                _other = _other.without_local()
+
+            if not _this.is_postrelease() and _other.is_postrelease():
+                # allow weak equality to allow `3.0.0-1` for `<=3.0.0`
+                _other = _other.without_postrelease()
+
+            if _other > _this:
                 return False
 
-            if not self._include_max and other == self.full_max:
+            if not self._include_max and _other == _this:
                 return False
 
         return True
 
-    def allows_all(self, other):  # type: ("VersionTypes") -> bool
-        from .version import Version
+    def allows_all(self, other: VersionConstraint) -> bool:
+        from poetry.core.semver.version import Version
 
         if other.is_empty():
             return True
@@ -99,13 +106,13 @@ class VersionRange(VersionConstraint):
         if isinstance(other, VersionUnion):
             return all([self.allows_all(constraint) for constraint in other.ranges])
 
-        if isinstance(other, VersionRange):
+        if isinstance(other, VersionRangeConstraint):
             return not other.allows_lower(self) and not other.allows_higher(self)
 
-        raise ValueError("Unknown VersionConstraint type {}.".format(other))
+        raise ValueError(f"Unknown VersionConstraint type {other}.")
 
-    def allows_any(self, other):  # type: ("VersionTypes") -> bool
-        from .version import Version
+    def allows_any(self, other: VersionConstraint) -> bool:
+        from poetry.core.semver.version import Version
 
         if other.is_empty():
             return False
@@ -116,15 +123,15 @@ class VersionRange(VersionConstraint):
         if isinstance(other, VersionUnion):
             return any([self.allows_any(constraint) for constraint in other.ranges])
 
-        if isinstance(other, VersionRange):
+        if isinstance(other, VersionRangeConstraint):
             return not other.is_strictly_lower(self) and not other.is_strictly_higher(
                 self
             )
 
-        raise ValueError("Unknown VersionConstraint type {}.".format(other))
+        raise ValueError(f"Unknown VersionConstraint type {other}.")
 
-    def intersect(self, other):  # type: ("VersionTypes") -> "VersionTypes"
-        from .version import Version
+    def intersect(self, other: VersionConstraint) -> VersionConstraint:
+        from poetry.core.semver.version import Version
 
         if other.is_empty():
             return other
@@ -139,8 +146,8 @@ class VersionRange(VersionConstraint):
 
             return EmptyConstraint()
 
-        if not isinstance(other, VersionRange):
-            raise ValueError("Unknown VersionConstraint type {}.".format(other))
+        if not isinstance(other, VersionRangeConstraint):
+            raise ValueError(f"Unknown VersionConstraint type {other}.")
 
         if self.allows_lower(other):
             if self.is_strictly_lower(other):
@@ -170,6 +177,7 @@ class VersionRange(VersionConstraint):
             # Because we already verified that the lower range isn't strictly
             # lower, there must be some overlap.
             assert intersect_include_min and intersect_include_max
+            assert intersect_min is not None
 
             return intersect_min
 
@@ -178,8 +186,8 @@ class VersionRange(VersionConstraint):
             intersect_min, intersect_max, intersect_include_min, intersect_include_max
         )
 
-    def union(self, other):  # type: ("VersionTypes") -> "VersionTypes"
-        from .version import Version
+    def union(self, other: VersionConstraint) -> VersionConstraint:
+        from poetry.core.semver.version import Version
 
         if isinstance(other, Version):
             if self.allows(other):
@@ -197,7 +205,7 @@ class VersionRange(VersionConstraint):
 
             return VersionUnion.of(self, other)
 
-        if isinstance(other, VersionRange):
+        if isinstance(other, VersionRangeConstraint):
             # If the two ranges don't overlap, we won't be able to create a single
             # VersionRange for both of them.
             edges_touch = (
@@ -230,8 +238,8 @@ class VersionRange(VersionConstraint):
 
         return VersionUnion.of(self, other)
 
-    def difference(self, other):  # type: ("VersionTypes") -> "VersionTypes"
-        from .version import Version
+    def difference(self, other: VersionConstraint) -> VersionConstraint:
+        from poetry.core.semver.version import Version
 
         if other.is_empty():
             return self
@@ -256,10 +264,11 @@ class VersionRange(VersionConstraint):
                 VersionRange(self.min, other, self.include_min, False),
                 VersionRange(other, self.max, False, self.include_max),
             )
-        elif isinstance(other, VersionRange):
+        elif isinstance(other, VersionRangeConstraint):
             if not self.allows_any(other):
                 return self
 
+            before: VersionConstraint | None
             if not self.allows_lower(other):
                 before = None
             elif self.min == other.min:
@@ -269,6 +278,7 @@ class VersionRange(VersionConstraint):
                     self.min, other.min, self.include_min, not other.include_min
                 )
 
+            after: VersionConstraint | None
             if not self.allows_higher(other):
                 after = None
             elif self.max == other.max:
@@ -282,6 +292,7 @@ class VersionRange(VersionConstraint):
                 return EmptyConstraint()
 
             if before is None:
+                assert after is not None
                 return after
 
             if after is None:
@@ -289,8 +300,8 @@ class VersionRange(VersionConstraint):
 
             return VersionUnion.of(before, after)
         elif isinstance(other, VersionUnion):
-            ranges = []  # type: List[VersionRange]
-            current = self
+            ranges: list[VersionRangeConstraint] = []
+            current: VersionRangeConstraint = self
 
             for range in other.ranges:
                 # Skip any ranges that are strictly lower than [current].
@@ -311,6 +322,7 @@ class VersionRange(VersionConstraint):
                     ranges.append(difference.ranges[0])
                     current = difference.ranges[-1]
                 else:
+                    assert isinstance(difference, VersionRangeConstraint)
                     current = difference
 
             if not ranges:
@@ -318,66 +330,13 @@ class VersionRange(VersionConstraint):
 
             return VersionUnion.of(*(ranges + [current]))
 
-        raise ValueError("Unknown VersionConstraint type {}.".format(other))
+        raise ValueError(f"Unknown VersionConstraint type {other}.")
 
-    def allows_lower(self, other):  # type: (VersionRange) -> bool
-        if self.min is None:
-            return other.min is not None
+    def flatten(self) -> list[VersionRangeConstraint]:
+        return [self]
 
-        if other.min is None:
-            return False
-
-        if self.min < other.min:
-            return True
-
-        if self.min > other.min:
-            return False
-
-        return self.include_min and not other.include_min
-
-    def allows_higher(self, other):  # type: (VersionRange) -> bool
-        if self.full_max is None:
-            return other.max is not None
-
-        if other.full_max is None:
-            return False
-
-        if self.full_max < other.full_max:
-            return False
-
-        if self.full_max > other.full_max:
-            return True
-
-        return self.include_max and not other.include_max
-
-    def is_strictly_lower(self, other):  # type: (VersionRange) -> bool
-        if self.full_max is None or other.min is None:
-            return False
-
-        if self.full_max < other.min:
-            return True
-
-        if self.full_max > other.min:
-            return False
-
-        return not self.include_max or not other.include_min
-
-    def is_strictly_higher(self, other):  # type: (VersionRange) -> bool
-        return other.is_strictly_lower(self)
-
-    def is_adjacent_to(self, other):  # type: (VersionRange) -> bool
-        if self.max != other.min:
-            return False
-
-        return (
-            self.include_max
-            and not other.include_min
-            or not self.include_max
-            and other.include_min
-        )
-
-    def __eq__(self, other):  # type: (Any) -> int
-        if not isinstance(other, VersionRange):
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, VersionRangeConstraint):
             return False
 
         return (
@@ -387,19 +346,19 @@ class VersionRange(VersionConstraint):
             and self._include_max == other.include_max
         )
 
-    def __lt__(self, other):  # type: (VersionRange) -> int
+    def __lt__(self, other: VersionRangeConstraint) -> bool:
         return self._cmp(other) < 0
 
-    def __le__(self, other):  # type: (VersionRange) -> int
+    def __le__(self, other: VersionRangeConstraint) -> bool:
         return self._cmp(other) <= 0
 
-    def __gt__(self, other):  # type: (VersionRange) -> int
+    def __gt__(self, other: VersionRangeConstraint) -> bool:
         return self._cmp(other) > 0
 
-    def __ge__(self, other):  # type: (VersionRange) -> int
+    def __ge__(self, other: VersionRangeConstraint) -> bool:
         return self._cmp(other) >= 0
 
-    def _cmp(self, other):  # type: (VersionRange) -> int
+    def _cmp(self, other: VersionRangeConstraint) -> int:
         if self.min is None:
             if other.min is None:
                 return self._compare_max(other)
@@ -408,16 +367,17 @@ class VersionRange(VersionConstraint):
         elif other.min is None:
             return 1
 
-        result = self.min._cmp(other.min)
-        if result != 0:
-            return result
+        if self.min > other.min:
+            return 1
+        elif self.min < other.min:
+            return -1
 
         if self.include_min != other.include_min:
             return -1 if self.include_min else 1
 
         return self._compare_max(other)
 
-    def _compare_max(self, other):  # type: (VersionRange) -> int
+    def _compare_max(self, other: VersionRangeConstraint) -> int:
         if self.max is None:
             if other.max is None:
                 return 0
@@ -426,16 +386,17 @@ class VersionRange(VersionConstraint):
         elif other.max is None:
             return -1
 
-        result = self.max._cmp(other.max)
-        if result != 0:
-            return result
+        if self.max > other.max:
+            return 1
+        elif self.max < other.max:
+            return -1
 
         if self.include_max != other.include_max:
             return 1 if self.include_max else -1
 
         return 0
 
-    def __str__(self):  # type: () -> str
+    def __str__(self) -> str:
         text = ""
 
         if self.min is not None:
@@ -446,17 +407,18 @@ class VersionRange(VersionConstraint):
             if self.min is not None:
                 text += ","
 
-            text += "{}{}".format("<=" if self.include_max else "<", self.max.text)
+            op = "<=" if self.include_max else "<"
+            text += f"{op}{self.max.text}"
 
         if self.min is None and self.max is None:
             return "*"
 
         return text
 
-    def __repr__(self):  # type: () -> str
-        return "<VersionRange ({})>".format(str(self))
+    def __repr__(self) -> str:
+        return f"<VersionRange ({str(self)})>"
 
-    def __hash__(self):  # type: () -> int
+    def __hash__(self) -> int:
         return (
             hash(self.min)
             ^ hash(self.max)
