@@ -1,7 +1,14 @@
-from poetry.core.semver import Version
-from poetry.core.semver import VersionUnion
-from poetry.core.semver import parse_constraint
+from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
+from poetry.core.semver.helpers import parse_constraint
+from poetry.core.semver.version import Version
+from poetry.core.semver.version_union import VersionUnion
+
+
+if TYPE_CHECKING:
+    from poetry.core.semver.version_constraint import VersionConstraint
 
 PYTHON_VERSION = [
     "2.7.*",
@@ -14,26 +21,26 @@ PYTHON_VERSION = [
     "3.6.*",
     "3.7.*",
     "3.8.*",
+    "3.9.*",
+    "3.10.*",
 ]
 
 
-def format_python_constraint(constraint):
+def format_python_constraint(constraint: VersionConstraint) -> str:
     """
     This helper will help in transforming
     disjunctive constraint into proper constraint.
     """
     if isinstance(constraint, Version):
         if constraint.precision >= 3:
-            return "=={}".format(str(constraint))
+            return f"=={str(constraint)}"
 
         # Transform 3.6 or 3
         if constraint.precision == 2:
             # 3.6
-            constraint = parse_constraint(
-                "~{}.{}".format(constraint.major, constraint.minor)
-            )
+            constraint = parse_constraint(f"~{constraint.major}.{constraint.minor}")
         else:
-            constraint = parse_constraint("^{}.0".format(constraint.major))
+            constraint = parse_constraint(f"^{constraint.major}.0")
 
     if not isinstance(constraint, VersionUnion):
         return str(constraint)
